package
{
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;

	public class MapData
	{		
		[Embed(source="data/spritesheet.png")]
		public static var ImgTiles:Class;
		[Embed(source="data/glitter.png")]
		public static var ImgAura:Class;
		
		public static const screenwidth:int=352;
		public static const screenheight:int=240;
		public static const tilewidth:int=16;
		public static const tileheight:int=15;
		public static const mapwidth:int=screenwidth/tilewidth;
		public static const mapheight:int=screenheight/tileheight;
		
		public static const spritetileoffset:int=4;
		public static const entcount:int=3;
		
		public var finished:Boolean=false;
		public var visibleentities:int=0;
		public var hp:int=5;
		public var level:int=0;
		/*
		tiles:
		0	blank
		1	wall
		2	fog
		*/
		
		public var tiles:Vector.<Vector.<int>>;
		public var aura:Vector.<Vector.<int>>;
		public var auratiles:Vector.<Vector.<int>>;
		
		public var entities:Vector.<Entity>;
	
		private var connectionmap:Vector.<Vector.<int>>;
		
		public function UpdateConnectionMap():void
		{
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					if (tiles[i][j]==1)
					{
						connectionmap[i][j]=-1;
					}
					else
					{
						connectionmap[i][j]=i*mapheight+j;
					}
					
				}
			}			
			
			var changed:Boolean=true;
			while (changed)
			{
				changed=false;
			
				for (i=1;i<mapwidth-1;i++)
				{
					for (j=1;j<mapheight-1;j++)
					{
						if (connectionmap[i][j]==-1)
							continue;
						
						var f:Function = function(dx:int,dy:int):void{
								if (connectionmap[i+dx][j+dy]!=-1 && connectionmap[i+dx][j+dy]>connectionmap[i][j])
								{
									changed=true;
									connectionmap[i+dx][j+dy]=connectionmap[i][j];
								}
							};
						
						f(1,0);
						f(0,-1);
						f(-1,0);
						f(0,1);
					}
				}
			}	
			RelabelSectors();
		}
		
		public function ConnectSectors():void
		{
			UpdateConnectionMap();
			trace("ConnectionCount pre " + ConnectionCount());
			
			while (Connected()==false)
			{
				ConnectFirstTwoSectors();
				UpdateConnectionMap();
				trace("current connection count " + ConnectionCount());
			}
			trace("ConnectionCount post " + ConnectionCount());
		}
		
		public function ConnectFirstTwoSectors()
		{
			var pointsA:Vector.<int> = new Vector.<int>();
			var pointsB:Vector.<int> = new Vector.<int>();
			
			for (var i:int=1;i<mapwidth-1;i++)
			{
				for (var j:int=1;j<mapheight-1;j++)
				{
					switch (connectionmap[i][j])
					{
						case 0:
							pointsA.push(i);
							pointsA.push(j);
							break;
						case 1:
							pointsB.push(i);
							pointsB.push(j);
							break;
					}
				}				
			}
			var closest:Vector.<int> = GetClosestTwoPoints(pointsA,pointsB);
			
			CarveLine(closest[0],closest[1],closest[2],closest[3]);
		}
		
		private function CarveLine(x1:int,y1:int,x2:int,y2:int):void
		{
			var curx:int=x1;
			var cury:int=y1;
			while (curx!= x2 || cury!=y2)
			{
				if (curx<x2)
				{
					curx++;
				}
				else if (cury<y2)
				{
					cury++;
				}
				else if (curx>x2)
				{
					curx--;
				}
				else if (cury>y2)
				{
					cury--;
				}
				tiles[curx][cury]=0;
			}
		}
		
		private function GetClosestTwoPoints(pA:Vector.<int>,pB:Vector.<int>):Vector.<int>
		{
			var closestdistsq:int=100000;
			var closestindexa:int=-1;
			var closestindexb:int=-1;
			for (var i:int=0;i<pA.length;i+=2)
			{
				for (var j:int=0;j<pB.length;j+=2)
				{
					var dx:int = pA[i]-pB[j];
					var dy:int = pA[i+1]-pB[j+1];
					var distsq:int = dx*dx+dy*dy;
					if (distsq<closestdistsq)
					{
						closestdistsq=distsq;
						closestindexa=i;
						closestindexb=j;
					}
				}
			}
			
			return new <int>[pA[closestindexa],pA[closestindexa+1],pB[closestindexb],pB[closestindexb+1]];
		}
		
		public function RelabelSectors():void
		{
			var sectorlist:Vector.<int> = new Vector.<int>();
			for (var i:int=1;i<mapwidth-1;i++)
			{
				for (var j:int=1;j<mapheight-1;j++)
				{
					if (connectionmap[i][j]!=-1)
					{
						if (sectorlist.indexOf(connectionmap[i][j])==-1)
						{
							sectorlist.push(connectionmap[i][j]);
						}
					}
				}
			}
			
			for (var i:int=1;i<mapwidth-1;i++)
			{
				for (var j:int=1;j<mapheight-1;j++)
				{
					connectionmap[i][j]=sectorlist.indexOf(connectionmap[i][j]);
				}
			}
		}
		
		public function ConnectionCount():int
		{
			var sectorlist:Vector.<int> = new Vector.<int>();
			for (var i:int=1;i<mapwidth-1;i++)
			{
				for (var j:int=1;j<mapheight-1;j++)
				{
					if (connectionmap[i][j]!=-1)
					{
						if (sectorlist.indexOf(connectionmap[i][j])==-1)
						{
							sectorlist.push(connectionmap[i][j]);
						}
					}
				}
			}
			return sectorlist.length;
		}
		
		public function Connected():Boolean
		{
			return ConnectionCount()==1;
		}
		
		public static function Init(_level:int,_hp:int):MapData
		{
			var mapdat:MapData = new MapData();
			mapdat.level=_level;
			mapdat.hp=_hp;
			
			var density:Number = 0.7+Math.random()*0.25;
			
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					mapdat.tiles[i][j] = 
						 (i==0||j==0||i==(mapwidth-1)||j==(mapheight-1)) ?	1
						: Math.random()<density ? 0
						:1;
					
				}
			}
			mapdat.tiles[mapwidth-4][mapheight-4]=2;
			mapdat.tiles[mapwidth-5][mapheight-4]=0;
			mapdat.tiles[mapwidth-3][mapheight-4]=0;
			mapdat.tiles[mapwidth-4][mapheight-5]=0;
			mapdat.tiles[mapwidth-4][mapheight-3]=0;
			
			mapdat.tiles[3+0][3+0]=0;
			mapdat.tiles[3+1][3+0]=0;
			mapdat.tiles[3-1][3+0]=0;
			mapdat.tiles[3+0][3+1]=0;
			mapdat.tiles[3+0][3-1]=0;
			
			mapdat.entities.push(Entity.Read("0,3,3"));//+int(Math.random()*(mapwidth-2)+1)+","+int(Math.random()*(mapheight-2)+1)));
			
			for (i=0;i<4+2*mapdat.level;i++)
			{
				var et:int;
				if (mapdat.level==0)
				{
					et=1;
				}
				else
				{
					switch(mapdat.level%3)
					{
						case 0:
							et = Math.random()<0.5 ? 1 : 2;
							break;
						case 1:
							et = Math.random()<0.25 ? 1 : 2;
							break;
						case 2:
							et = Math.random()<0.75 ? 1 : 2;
							break;
					}
				}
				
				var ex:int=int(Math.random()*(mapwidth-2))+1;
				var ey:int=int(Math.random()*(mapheight-2))+1;
				if (ex==3&&ey==3)
				{
					ex+=int(2+Math.random()*5);
					ey+=int(2+Math.random()*5);
				}
				
				if (mapdat.tiles[ex][ey]==2)
				{
					ex-=1;
					ey-=1;
				}
				
				mapdat.entities.push(Entity.Read(et +"," + ex + "," + ey));
				
				
				if (mapdat.tiles[ex][ey]==1)
				{
					mapdat.tiles[ex][ey]=0;
				}
			
			}
			
			
			mapdat.ConnectSectors();
						
			mapdat._calculateaura();
			return mapdat;
		}
		
		public function MapData()
		{
			tiles = new Vector.<Vector.<int>>();
			aura = new Vector.<Vector.<int>>();
			auratiles = new Vector.<Vector.<int>>();
			connectionmap = new Vector.<Vector.<int>>();
			
			for (var i:int=0;i<mapwidth;i++)
			{
				var r:Vector.<int> = new Vector.<int>();
				var r2:Vector.<int> = new Vector.<int>();
				var r3:Vector.<int> = new Vector.<int>();
				var r4:Vector.<int> = new Vector.<int>();
				
				for (var j:int=0;j<mapheight;j++)
				{
					r.push(int(Math.random()*spritetileoffset));
					r2.push(int(Math.random()*spritetileoffset));
					r3.push(int(Math.random()*spritetileoffset));
					r4.push(0);
				}
				tiles.push(r);
				aura.push(r2);
				auratiles.push(r3);
				connectionmap.push(r4);
			}
			
			entities = new Vector.<Entity>();
			finished=false;
		}
		
		private function clearaura():void
		{			
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					aura[i][j]=0;
				}
			}				
		}
		
		private const drawradius:int=2;
		
		private function drawAuraDiskAround(e:Entity,p:Entity):void
		{
			var cr:Number = Math.sqrt(Math.random()*drawradius);
			var cx:int = e.x;
			var cy:int = e.y;
			
			var l:int=Math.max(cx-drawradius,0);
			var r:int=Math.min(cx+drawradius,mapwidth-1);
			var t:int=Math.max(cy-drawradius,0);
			var b:int=Math.min(cy+drawradius,mapheight-1);
			if (p.x>=l && p.x<=r &&
				p.y>=t && p.y<=b)
			{				
				visibleentities++;
				var l2:int=Math.max(p.x-drawradius,0);
				var r2:int=Math.min(p.x+drawradius,mapwidth-1);
				var t2:int=Math.max(p.y-drawradius,0);
				var b2:int=Math.min(p.y+drawradius,mapheight-1);
				for (var i:int=l2;i<=r2;i++)
				{
					for (var j:int=t2;j<=b2;j++)
					{
						aura[i][j]++;
					}
				}
			}
		}
		
		private function drawAuraCrossAround(e:Entity,p:Entity):void
		{
			if(e.y==p.y)
			{
				visibleentities++;
				for (var i:int=0;i<mapwidth;i++)
				{
					aura[i][e.y]++;
				}
			}
			else if (e.x==p.x)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					aura[e.x][j]++;
				}
			}
		}
		
		private function _calculateaura():void
		{
			clearaura();
			
			visibleentities=0;
			
			var p:Entity = getplayer();
			//populate aura
			for (var i:int=0;i<entities.length;i++)
			{
				var e:Entity = entities[i];
				switch (e.type)
				{
					case 0:
						break;
					default:
						if (e.type%2==1)
						{
							drawAuraCrossAround(e,p);
						}
						else
						{
							drawAuraDiskAround(e,p);
						}
						break;
				}
			}
			//generate tiles
			
			for (i=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					/*
					var val_left:int = i>0 && aura[i-1][j]>0 ? 1 : 0;
					var val_right:int = i<mapwidth-1 && aura[i]+1[j]>0 ? 1 : 0;
					var val_up:int = j>0 && aura[i][j-1]>0 ? 1 : 0;
					var val_down:int = j<mapheight-1 && aura[i][j+1]>0 ? 1 : 0;*/
					
					auratiles[i][j] = aura[i][j]>0 ? 1 : 0;
				}
			}
			
				
		}
		
		public static function Read(dat:String):MapData
		{
			var split:Array = dat.split("|");
			
			var mapdat:MapData = new MapData();
			
			var mapsplit:Array = split[0].split(",");
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					mapdat.tiles[i][j]=mapsplit[i*mapwidth+j];
				}
			}
			
			for (i=1;i<split.length;i++)
			{
				var entstr:String = split[i];
				var e:Entity = Entity.Read(entstr);
				mapdat.entities.push(e);
			}
			
			return mapdat;
		}
		
		private function DamagePlayer():void
		{
			hp--;
		}
		
		private var movear:Array=[[-1,0],[1,0],[0,-1],[0,1]];
		public function trymove(dx:int,dy:int):MapData
		{
			var mapdat:MapData = Copy();
			
			if (finished)
				return mapdat;
			
			var pi:int=playerindex();
			mapdat._trymove(pi,dx,dy);
			var p:Entity = mapdat.getplayer();			
			if (mapdat.tiles[p.x][p.y]==2)
			{
				mapdat.finished=true;
			}
			
			for (var i:int=1;i<mapdat.entities.length;i++)
			{
				var e:Entity=entities[i];
				if (mapdat.NearPlayer(e) && mapdat.finished==false)
				{
					mapdat.DamagePlayer();
				}
				else
				{
					var rd:int = int(Math.random()*movear.length);
					mapdat._trymove(i,movear[rd][0],movear[rd][1]);
				}
			}
			mapdat._calculateaura();
			
			
			
			return mapdat;
		}
		
		public function canmoveto(x:int,y:int):Boolean
		{
			if (tiles[x][y]==1)
				return false;
			for (var i:int=0;i<entities.length;i++)
			{
				if (entities[i].x==x && entities[i].y==y)
				{
					return false;
				}
			}
			return true;
		}
		
		private function _trymove(entityindex:int,dx:int,dy:int):void
		{
			var ent:Entity = entities[entityindex];
			var tx:int=ent.x+dx;
			var ty:int=ent.y+dy;
			if (canmoveto(tx,ty))
			{
				ent.x=tx;
				ent.y=ty;
			}
			else
			{
				if (entityindex==0)
				{
					var e:int = GetEntAt(tx,ty);
					if (e>=0)
					{
					//	entities.splice(e,1);
					//	hp++;
					//	trace("removing"+e);
					}
				}
			}
		}
		
		private function GetEntAt(x:int,y:int):int
		{
			for (var i:int=0;i<entities.length;i++)
			{
				var e:Entity=entities[i];
				if (e.x==x&&e.y==y)
					return i;
			}
			return -1;
		}
		
		private function playerindex():int
		{
			return 0;
		}
		
		private function getplayer():Entity
		{
			for (var i:int=0;i<entities.length;i++)
			{
				if (entities[i].type==0)
				{
					return entities[i];
				}
			}
			trace("PLAYER NOT FOUND OH NOES");
			return null;
		}
		
		public function Write():String
		{
			var result:String="";

			for (i=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					result+=tiles[i][j]	+",";
				}
			}
			for (var i:int=0;i<entities.length;i++)
			{
				result+="|";
				result+=entities[i].Write();
			}
			return result;
		}
		
		
		public function Copy():MapData
		{
			var map:MapData = new MapData();
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					map.tiles[i][j]=tiles[i][j];
				}
			}
			
			for (i=0;i<entities.length;i++)
			{
				map.entities.push(entities[i].Copy());
			}
			map.tilemapdirty=tilemapdirty;
			map.finished=finished;
			map.hp=hp;
			return map;
		}
		
		private function NearPlayer(e:Entity):Boolean
		{
			var p:Entity = getplayer();
			if (Math.abs(p.x-e.x)<1.5 && p.y==e.y)
				return true;
			if (Math.abs(p.y-e.y)<1.5 && p.x==e.x)
				return true;
			return false;				
		}
		public var tilemapdirty:Boolean=true;
		public function Draw(bgtiles:FlxGroup,entitytiles:FlxGroup,fgtiles:FlxGroup,auralayer:FlxGroup):void
		{
			var s:FlxSprite;
			
			
			if (tilemapdirty)
			{
				tilemapdirty=false;
				bgtiles.members=[];				
				fgtiles.members=[];
				for (var i:int=0;i<mapwidth;i++)
				{
					for (var j:int=0;j<mapheight;j++)
					{
						s = new FlxSprite(i*tilewidth,j*tileheight);
						if (tiles[i][j]<spritetileoffset-1)
						{
							s.loadGraphic(ImgTiles,true,false,16,15);					
							s.setFrame(tiles[i][j]);
							bgtiles.add(s);
						}
						else
						{
							s.loadGraphic(ImgTiles,true,false,16,15);					
							s.setFrame(0);
							bgtiles.add(s);			
							
							s = new FlxSprite(i*tilewidth,j*tileheight);
							s.loadGraphic(ImgTiles,true,false,16,15);					
							s.setFrame(tiles[i][j]);
							fgtiles.add(s);						
						}
					}
				}
			}
			
			entitytiles.members=[];
			
			for (i=0;i<entities.length;i++)
			{
				var e:Entity=entities[i];
				if (NearPlayer(e))
				{
					s = new FlxSprite(e.x*tilewidth,e.y*tileheight);
					s.loadGraphic(ImgTiles,true,false,16,15);					
					s.setFrame(e.type+spritetileoffset);
					entitytiles.add(s);
				}
			}
			
			auralayer.members=[];
			for (var i:int=0;i<mapwidth;i++)
			{
				for (var j:int=0;j<mapheight;j++)
				{
					if (auratiles[i][j]>0)
					{
						s = new FlxSprite(i*tilewidth,j*tileheight);
						s.loadGraphic(ImgAura,true,false,16,15);
						//s.addAnimation("default",[0,1,2,3],5,true);
//						s.play("default");
						s.setFrame(int(Math.random()*4));
						auralayer.add(s);
					}
				}				
			}
			
		}
		
	}
}